/*
 * $Id: Namespace.h 2504 2009-02-23 13:16:02Z karijes $
 *
 * Copyright (c) 2005-2007 edelib authors
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

/* 
 * this file is here just to allow doxygen to see
 * pickup namespace information 
 */

/**
 * \namespace edelib
 * This is a main namespace where all edelib components laid.
 * Also namespace usage can be optional, and disabled during
 * compilation phase.
 */
