/*
 * $Id: Version.h.in 2839 2009-09-28 11:36:20Z karijes $
 *
 * Version info
 * Copyright (c) 2005-2007 edelib authors
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __EDELIB_VERSION_H__
#define __EDELIB_VERSION_H__

#define EDELIB_MAJOR_VERSION	2
#define EDELIB_MINOR_VERSION	1
#define EDELIB_PATCH_VERSION	0

#define EDELIB_VERSION          "1.0"

#endif
