/* generated file; please modify configure.in */
#ifndef __EDELIB_EDELIB_CONFIG_H__
#define __EDELIB_EDELIB_CONFIG_H__

#define EDELIB_HAVE_ISO_VARARGS 1
#define EDELIB_HAVE_DBUS 1
#define EDELIB_HAVE_FLTK_XFT 1
#define EDELIB_INSTALL_PREFIX "/usr/local"
#define EDELIB_HAVE_RTTI 1

#endif
